#!/bin/sh
#
# Copyright (c) 1998 The University of Utah. All rights reserved.
#
# See the file "license.terms" for information on usage and redistribution
# of this file.
#
# Contributed by the Flux Research Group at the University of Utah.
# Authors: Leigh Stoller
#

# This script builds a multiboot image from Kaffe plus other stuff.  The
# multiboot image contains the files specified, and results in a memory based
# filesystem being created by the oskit kernel at boot time. By default, the
# output file name is called "Image." Like the other scripts, there are some
# environment specific options that need to be changed, either by copying the
# script and editing it, or by passing them in as command line options.
# Please look at the first part of the script to see what needs to be
# changed.
# 
# The most important arguments to mkimage.sh are the -dir args, which specify
# which directories to place into the multiboot image. You can specify as
# many -dir args as you like. The minimum usage would be:
# 
# 	mkimage.sh -dir /tmp/foo -dir /tmp/fee
# 
# which will add the contents of those two directories (and all their
# subdirs/files) to the image.  By default, you alway get the contents of the
# Kaffe install/share/kaffe directory, which includes the standard kaffe
# class files Klasses.jar and pizza.jar.
# 
# The other important consideration is the default CLASSPATH. Since there is
# no "easy" way to pass a long environment string to an oskit kernel, the
# oskit/kaffe startup code will read a file from /etc that contains the
# default classpath. This file is called /etc/kaffe_classpath, and is the
# usual colon separated list of names. This script will add that file to the
# boot image. Note that the target file resides in /etc, but the source file
# can reside anywhere, such as the current directory where this script is
# executed. It defaults to the value of "kaffe_classpath", but can be specified
# as a command line option to this script using the --classpathfile= option,
# or by changing the CLASSPATHFILE variable in the mkimage.sh script.
# 
# *** What should the CLASSPATH be? When the multiboot image is booted, the
# filesystem it sets up will have the same layout as all the directories
# given with the -dir option, plus the default Kaffe install/share/kaffe
# directory. In addition, the current working directory of the OSKit kernel
# will be the root directory. So, in the example above, the filesystem will
# contain /tmp/foo (and all its subdirs/files), /tmp/fee (and all its
# subdirs/files), and $prefix/share/kaffe (and all its subdirs/files).
# Therefore, the CLASSPATH in your classpathfile might look like:
# 
# 	.:/tmp/foo:/tmp/fee:$prefix/share/kaffe/Klasses.jar
# 
# where $prefix is expanded out, not a shell variable. There is an example
# kaffe_classpath file in this (config/i386/oskit) directory.
#
# *** Can the initial current working directory be changed? Yes, you can change
# the initial working directory of Kaffe by specifying a CWD option when the
# boot image is invoked. Options of the form FOO=bar result in the creation
# of an environment variable FOO. When Kaffe starts up, it will look for the
# CWD environment variable, and if it exists, chdir to that location.
# For example:
#
#	NetBoot> fuggles:/z/tmp/Image CWD=/tmp/foo -- HelloWorldApp
# 
# *** Can a different classpathfile other than /etc/kaffe_classpath be
# specified? Yes, when you start the oskit, you can change the name of the
# classpathfile to any other file in the multiboot image by using the
# environment variable command line option. That is, you can add
# CLASSPATHFILE=some_file_name to the oskit the command line, and that
# becomes an environment variable that the oskit/kaffe startup code will look
# for before attempting to open the default file /etc/kaffe_classpath.
#

# Your Kaffe object dir
OBJDIR=/y/stoller/kaffe-oskit/obj

# Your Kaffe prefix (install) directory
PREFIX=/y/stoller/kaffe-oskit/install

# where your oskit was installed
OSKITDIR=/y/stoller/oskit-real/install

# Default classpath file
CLASSPATHFILE=kaffe_classpath

# The appropriate OSKIT boot image constructor script. Default to Multiboot.
MKBOOTIMAGE=$OSKITDIR/bin/mkmbimage

# A list of directories
DIRS=

# Parse the command-line options to override above.
until [ $# -eq 0 ]
do
	case "$1" in
		--objdir=* )
			OBJDIR=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;
			 shift;;
		--prefix=* )
			PREFIX=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;
			shift;;
		--oskitdir=* )
			OSKITDIR=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;
			shift;;
		--classpathfile=* )
			CLASSPATHFILE=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;
			shift;;
		--mkbootimage=* )
			MKBOOTIMAGE=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;
			shift;;
		-dir )
			DIRS="$DIRS $2" ;
			shift;
			shift;;
		* ) echo "Bad args";
			  exit 1 ;;
	esac
done

# The Kaffe "kernel"
KAFFE=$OBJDIR/kaffe/kaffe/Kaffe

# The directory with the minimum necessary class files.
CLASSDIR="$PREFIX/share/kaffe"

# The final list of directories
DIRS="$CLASSDIR $DIRS"

# The place where the default classpath will be created.
CPF="$CLASSPATHFILE:/etc/kaffe_classpath"

for DIR in $DIRS
do
	for FILE in `find $DIR -type f -print`
	do
	    echo "$FILE:$FILE"
	done
done | $MKBOOTIMAGE -o $OBJDIR/Image -stdin $KAFFE $CPF

